"""
Module contains pydantic model for events meta
"""
import re

from ..db.user_defined_filters import META_OPERATOR_MAP
from ..web.validators import validateMetaFilter
from .bases import JsonBaseModel, isPydanticFirst

if isPydanticFirst:  # TODO: remove after LUNA-6371 completed  # pylint: disable-msg=fixme
    from pydantic import Extra, root_validator
else:
    from pydantic import ConfigDict, model_validator  # pylint: disable-msg=no-name-in-module

    from ..errors.exception import PydanticError as Error
META_RX = re.compile(r"^(?P<path>(\w+[.])*([a-zA-Z0-9]+[_]?)+)" rf"(__(?P<operator>({'|'.join(META_OPERATOR_MAP)})))?$")

if isPydanticFirst:  # TODO: remove after LUNA-6371 completed  # pylint: disable-msg=fixme
    validator = root_validator
    validatorOptions = {"pre": False, "skip_on_failure": True}
else:
    validator = model_validator
    validatorOptions = {"mode": "before"}


if isPydanticFirst:
    validator = root_validator
    validatorOptions = {"pre": False, "skip_on_failure": True}
else:
    validator = model_validator
    validatorOptions = {"mode": "before"}


class Meta(JsonBaseModel):
    """
    Meta validation class
    """

    if isPydanticFirst:  # TODO: remove after LUNA-6371 completed # pylint: disable-msg=fixme

        class Config:
            """Pydantic model config."""

            extra = Extra.allow

    else:
        model_config = ConfigDict(extra="allow")

    @validator(**validatorOptions)
    def mainValidator(cls, values):  # pylint: disable-msg=no-self-argument
        """
        Main validator checks:
            - meta validation
        Args:
            values: loaded values
        Returns:
            loaded values
        """
        if not isPydanticFirst:  # TODO: remove after LUNA-6371 completed # pylint: disable-msg=fixme
            if not isinstance(values, dict):
                raise Error.PydanticInvalidDictError()  # pylint: disable-msg=used-before-assignment

        if values:
            try:
                for rawname, rawvalue in values.items():
                    parsed = META_RX.match(rawname)
                    if not parsed:
                        if isPydanticFirst:  # TODO: remove after LUNA-6371 completed # pylint: disable-msg=fixme
                            raise ValueError("Wrong filter name")
                        raise Error.PydanticValueError.format("Wrong filter name")()
                    validateMetaFilter(rawvalue, parsed.group("operator") or "eq")
            except Exception as exc:
                if isPydanticFirst:  # TODO: remove after LUNA-6371 completed # pylint: disable-msg=fixme
                    raise ValueError(f"Validation of filter '{rawname}' has failed. {exc.args[0]}") from exc

                raise Error.PydanticValueError.format(
                    f"Validation of filter '{rawname}' has failed. {exc.args[1]}"
                )() from exc

        return values

    def asDict(self) -> dict:
        """
        Overrides super.asDict to prevent using alias for meta filters keys
        """
        if isPydanticFirst:  # TODO: remove after LUNA-6371 completed # pylint: disable-msg=fixme
            return self.__dict__
        return self.model_dump()
