"""Utils for working with images."""
import base64

from crutches_on_wheels.cow.errors.errors import Error
from crutches_on_wheels.cow.errors.exception import VLException

BASE64_TO_DATA = {
    "image/x-jpeg-base64": "image/jpeg",
    "image/x-png-base64": "image/png",
    "image/x-bmp-base64": "image/bmp",
    "image/x-tiff-base64": "image/tiff",
    "image/x-portable-pixmap-base64": "image/x-portable-pixmap",
    "application/x-sdk-descriptor-base64": "application/x-sdk-descriptor",
    "application/x-vl-xpk-base64": "application/x-vl-xpk",
}


def convertToBytes(data: bytes, contentType: str) -> tuple[bytes, str]:
    """
    Convert data from base64 to bytes.

    Args:
        img: input data in base64 or bytes
        contentType: data content type

    Returns:
        binary data and content type
    Raises:
        VLException(Error.ConvertBase64Error, 400, isCriticalError=False): if failed convert to bytes from base64
    """

    try:
        newContentType = BASE64_TO_DATA[contentType]
    except KeyError:
        raise ValueError from None

    try:
        return base64.b64decode(data), newContentType
    except ValueError as e:
        raise VLException(Error.ConvertBase64Error, 400, isCriticalError=False) from e
